@@ -1,4 +1,4 @@
-/* $Id: cmd_user.h,v 1.27 2005/01/25 23:01:45 kuhlmann Exp $ */
+/* $Id: cmd_user.h,v 1.28 2005/05/24 21:27:50 kuhlmann Exp $ */
 
 #ifndef MICQ_USER_H
 #define MICQ_USER_H
@@ -19,20 +19,9 @@ struct jumpstr {
 
 typedef struct jumpstr jump_t;
 
-struct aliasstr {
-    char *name;
-    char *expansion;
-    struct aliasstr *next;
-};
-
-typedef struct aliasstr alias_t;
-	
-
 jump_t *CmdUserTable (void);
 jump_t *CmdUserLookup (const char *command);
 
-alias_t *CmdUserAliases (void);
-
 void CmdUser (const char *command);
 void CmdUserInput (strc_t line);
 void CmdUserInterrupt (void);
