@@ -1,11 +1,8 @@
-/* $Id: mreadline.h,v 1.18 2003/10/11 23:07:44 kuhlmann Exp $ */
+/* $Id: mreadline.h,v 1.19 2005/04/18 18:16:54 kuhlmann Exp $ */
 
 #ifndef MICQ_MREADLINE_H
 #define MICQ_MREADLINE_H
 
-#define Echo_Off()
-#define Echo_On()  
-
 void   rl_print  (const char *str);
 void   rl_printf (const char *str, ...) __attribute__ ((format (__printf__, 1, 2)));
 int    rl_pos (void);
