@@ -2,12 +2,6 @@
 #ifndef MICQ_UTIL_RL_H
 #define MICQ_UTIL_RL_H 1
 
-typedef struct rl_autoexpand_s {
-  const struct rl_autoexpand_s *next;
-  const char *command;
-  const char *replace;
-} rl_autoexpand_t;
-
 extern UDWORD rl_columns;
 extern volatile UBYTE rl_signal;
 
@@ -18,9 +12,6 @@ void ReadLineClrScr (void);
 void ReadLineHandleSig (void);
 void ReadLineAllowTab (UBYTE onoff);
 
-const rl_autoexpand_t *ReadLineListAutoExpand (void);
-void ReadLineAutoExpand (const char *command, const char *replace);
-
 str_t ReadLine (UBYTE newbyte);
 
 void ReadLinePrompt (void);
