@@ -1,4 +1,4 @@
-/* $Id: file_util.c,v 1.204 2005/04/24 11:15:42 kuhlmann Exp $ */
+/* $Id: file_util.c,v 1.208 2005/05/28 15:52:32 kuhlmann Exp $ */
 /* Copyright: This file may be distributed under version 2 of the GPL licence. */
 
 #include "micq.h"
@@ -29,6 +29,7 @@
 #include "buildmark.h"
 #include "util_ui.h"
 #include "file_util.h"
+#include "util_alias.h"
 #include "tabs.h"
 #include "contact.h"
 #include "tcp.h"
@@ -898,12 +899,17 @@ int Read_RC_File (FILE *rcf)
                 }
                 else if (!strcasecmp (cmd, "alias"))
                 {
+                    char autoexpand = FALSE;
+                    if (s_parsekey (&args, "auto") || s_parsekey (&args, "autoexpand"))
+                        autoexpand = TRUE;
                     PrefParse (tmp);
                     tmp = strdup (s_quote (tmp));
                     PrefParse (tmp2);
                     tmp2 = strdup (tmp2);
 
-                    CmdUser (cmd = strdup (s_sprintf ("\\alias %s %s", tmp, tmp2)));
+                    CmdUser (cmd = strdup (s_sprintf ("\\alias %s%s%s%s %s",
+                        autoexpand ? "autoexpand " : "",
+                        *tmp == '"' ? "" : "\"", tmp, *tmp == '"' ? "" : "\"", tmp2)));
                     
                     free (cmd);
                     free (tmp);
@@ -1522,7 +1528,7 @@ int PrefWriteStatusFile (void)
             if (cont->group && cont->group->serv == ss)
             {
                 ContactAlias *alias;
-                fprintf (stf, "entry %d %10ld %s", cont->id, cont->uin, s_quote (cont->nick));
+                fprintf (stf, "entry %d %10ld %s", 0, cont->uin, s_quote (cont->nick));
                 for (alias = cont->alias; alias; alias = alias->more)
                     fprintf (stf, " %s", s_quote (alias->alias));
                 fprintf (stf, "\n%s", OptString (&cont->copts));
@@ -1722,11 +1728,13 @@ int PrefWriteConfFile (void)
     fprintf (rcf, "# The aliases.\n");
     fprintf (rcf, "[Strings]\n");
     {
-        alias_t *node;
-        for (node = CmdUserAliases (); node; node = node->next)
+        const alias_t *node;
+        for (node = AliasList (); node; node = node->next)
         {
-            fprintf (rcf, "alias %s", s_quote (node->name));
-            fprintf (rcf, " %s\n", s_quote (node->expansion));
+            const char *tmp = s_quote (node->command);
+            fprintf (rcf, "alias %s%s%s%s", node->autoexpand ? "autoexpand " : "",
+                *tmp == '"' ? "" : "\"", tmp, *tmp == '"' ? "" : "\"");
+            fprintf (rcf, " %s\n", s_quote (node->replace));
         }
     }
     fprintf (rcf, "\n");
