@@ -1,4 +1,4 @@
-/* $Id: micq.c,v 1.150 2005/04/25 16:51:32 kuhlmann Exp $ */
+/* $Id: micq.c,v 1.152 2005/05/28 15:31:57 kuhlmann Exp $ */
 /* Copyright
  * This file may be distributed under version 2 of the GPL licence.
  */
@@ -47,6 +47,7 @@
 #include "oldicq_client.h"
 #include "util_tcl.h"
 #include "util_ssl.h"
+#include "util_alias.h"
 #include "os.h"
 
 #define MICQ_ICON_1 "   " GREEN "_" SGR0 "     "
@@ -290,8 +291,8 @@ static void Init (int argc, char *argv[])
     i18nInit (arg_i);
     PreferencesInit (prG);
     ReadLineInit ();
-    ReadLineAutoExpand ("r", "msg %r");
-    ReadLineAutoExpand ("a", "msg %a");
+    AliasAdd ("r", "msg %r", TRUE); /*"*/
+    AliasAdd ("a", "msg %a", TRUE);
     
     loaded = arg_h ? 0 : PrefLoad (prG);
     prG->verbose &= ~0x80000000UL;
