@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_base.c,v 1.6 2004/12/21 20:04:06 kuhlmann Exp $
+ * $Id: oscar_base.c,v 1.7 2005/04/18 18:16:51 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -310,9 +310,7 @@ void FlapCliIdent (Connection *conn)
         rl_print (i18n (2063, "You need to save your password in your ~/.micq/micqrc file.\n"));
 #else
         rl_printf ("%s ", i18n (1063, "Enter password:"));
-        Echo_Off ();
         pwd = UtilIOReadline (stdin);
-        Echo_On ();
         conn->passwd = strdup (pwd ? ConvFrom (pwd, prG->enc_loc)->txt : "");
 #endif
     }
