@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_icbm.c,v 1.23 2005/04/09 18:27:26 kuhlmann Exp $
+ * $Id: oscar_icbm.c,v 1.24 2005/06/04 20:53:26 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -63,9 +63,9 @@ JUMP_SNAC_F(SnacSrvIcbmerr)
     Connection *serv = event->conn;
     UWORD err = PacketReadB2 (event->pak);
 
-    if ((event->pak->ref & 0xffff) == 0x1771 && (err == 0xe || err == 0x4))
+    if ((event->pak->ref & 0xffff) == 0x1771 && (err == 0xe || err == 4 || err == 9))
     {
-        if (err == 0xe)
+        if (err == 0xe || err == 9)
             rl_print (i18n (2017, "The user is online, but possibly invisible.\n"));
         else
             rl_print (i18n (2564, "The user couldn't be detected to be online.\n"));
@@ -438,39 +438,41 @@ UBYTE SnacCliSendmsg2 (Connection *serv, Contact *cont, Opt *opt)
     PacketWriteB2 (pak, 2);
     PacketWriteCont (pak, cont);
     
+    PacketWriteTLV     (pak, 5);
+     PacketWrite2       (pak, 0);
+     PacketWriteB4      (pak, mtime);
+     PacketWriteB4      (pak, mid);
+
+     PacketWriteCapID   (pak, peek ? CAP_NONE : CAP_SRVRELAY);
+     PacketWriteTLV2    (pak, 10, 1);
+     PacketWriteB4      (pak, 0x000f0000); /* empty TLV(15) */
+     PacketWriteTLV     (pak, 10001);
+      PacketWriteLen     (pak);
+       PacketWrite2       (pak, serv->assoc && serv->assoc->connect & CONNECT_OK
+                              ? serv->assoc->version : 8);
+       PacketWriteCapID   (pak, CAP_NONE);
+       PacketWrite2       (pak, 0);
+       PacketWrite4       (pak, 3);
+       PacketWrite1       (pak, 0);
+       PacketWrite2       (pak, serv->our_seq_dc);
+      PacketWriteLenDone (pak);
+      SrvMsgAdvanced     (pak, serv->our_seq_dc, opt_type, serv->status, cont->status, -1, c_out_for (opt_text, cont, opt_type));
+      PacketWrite4       (pak, TCP_COL_FG);
+      PacketWrite4       (pak, TCP_COL_BG);
+      if (CONT_UTF8 (cont, opt_type))
+          PacketWriteDLStr     (pak, CAP_GID_UTF8);
+     PacketWriteTLVDone (pak);
+    PacketWriteTLVDone (pak);
+    PacketWriteB4      (pak, 0x00030000); /* empty TLV(3) */
+    
     if (peek)
-    {
+    {    
         PacketWriteB4  (pak, 0x00060000);
         SnacSend (serv, pak);
         OptD (opt);
     }
     else
     {
-         PacketWriteTLV     (pak, 5);
-         PacketWrite2       (pak, 0);
-         PacketWriteB4      (pak, mtime);
-         PacketWriteB4      (pak, mid);
-         PacketWriteCapID   (pak, CAP_SRVRELAY);
-         PacketWriteTLV2    (pak, 10, 1);
-         PacketWriteB4      (pak, 0x000f0000); /* empty TLV(15) */
-         PacketWriteTLV     (pak, 10001);
-          PacketWriteLen     (pak);
-           PacketWrite2       (pak, serv->assoc && serv->assoc->connect & CONNECT_OK
-                                  ? serv->assoc->version : 8);
-           PacketWriteCapID   (pak, CAP_NONE);
-           PacketWrite2       (pak, 0);
-           PacketWrite4       (pak, 3);
-           PacketWrite1       (pak, 0);
-           PacketWrite2       (pak, serv->our_seq_dc);
-          PacketWriteLenDone (pak);
-          SrvMsgAdvanced     (pak, serv->our_seq_dc, opt_type, serv->status, cont->status, -1, c_out_for (opt_text, cont, opt_type));
-          PacketWrite4       (pak, TCP_COL_FG);
-          PacketWrite4       (pak, TCP_COL_BG);
-          if (CONT_UTF8 (cont, opt_type))
-              PacketWriteDLStr     (pak, CAP_GID_UTF8);
-         PacketWriteTLVDone (pak);
-        PacketWriteTLVDone (pak);
-        PacketWriteB4      (pak, 0x00030000); /* empty TLV(3) */
         QueueEnqueueData (serv, QUEUE_TYPE2_RESEND, serv->our_seq_dc,
                           time (NULL), pak, cont, opt, &SnacCallbackType2);
     }
