@@ -30,7 +30,7 @@
  * As a special exemption, you may link your binary against OpenSSL,
  * provided your operating system distribution doesn't ship libgnutls.
  *
- * $Id: oscar_roster.c,v 1.19 2005/04/24 11:38:49 kuhlmann Exp $
+ * $Id: oscar_roster.c,v 1.22 2005/05/17 21:02:56 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -207,7 +207,7 @@ JUMP_SNAC_F(SnacSrvReplyroster)
         
         re->name = strdup (ConvFromServ (cname));
 
-        j = TLVGet (re->tlv, 305);
+        j = TLVGet (re->tlv, TLV_NICK);
         assert (j < 200 || j == (UWORD)-1);
         if (j != (UWORD)-1 && re->tlv[j].str.len)
             re->nick = strdup (ConvFromServ (&re->tlv[j].str));
@@ -217,39 +217,55 @@ JUMP_SNAC_F(SnacSrvReplyroster)
         rref = &roster->generic;
         switch (re->type)
         {
-            case 0: /* normal contact */
+            case ROSTER_TYPE_NORMAL:
+                /* TLV_REQAUTH */
+                /* TLV_NICK */
+                /* TLV_LOCALMAIL */
+                /* TLV_LOCALSMS */
+                /* TLV_LOCALCOMM */
+                /* TLV_LOCALACT */
+                /* TLV_LOCALSOUND */
+                /* TLV_LASTUPD */
                 rref = &roster->normal;
                 break;
-            case 1: /* group */
+            case ROSTER_TYPE_GROUP:
+                /* TLV_GROUPITEMS */
                 if (re->tag || re->id)
                     rref = &roster->groups;
                 break;
-            case 2: /* visible */
+            case ROSTER_TYPE_VISIBLE:
                 rref = &roster->visible;
                 break;
-            case 3: /* invisible */
+            case ROSTER_TYPE_INVISIBLE:
                 rref = &roster->invisible;
                 break;
-            case 14: /* ignore */
+            case ROSTER_TYPE_IGNORE:
                 rref = &roster->ignore;
                 break;
-            case 15: /* wierd */
-            case 17: /* wierd */
-            case 20: /* LastUpdateDate */
+            case ROSTER_TYPE_LASTUPD: /* LastUpdateDate */
+            case ROSTER_TYPE_WIERD17: /* wierd */
+            case ROSTER_TYPE_ICON: /* buddy icon */
+                /* TLV_ICON */
                 break;
-            case 19: /* ImportTime */
-                j = TLVGet (re->tlv, 212);
+            case ROSTER_TYPE_IMPORTT: /* ImportTime */
+                j = TLVGet (re->tlv, TLV_IMPORT);
                 if (j != (UWORD)-1 && re->tlv[j].str.len == 4)
                     roster->import = re->tlv[j].nr;
                 else
                     rl_printf ("#Bogus ImportTime item: %d: %s %d %d.\n", re->type, re->name, re->tag, re->id);
                 break;
-            case 9: /* ICQTIC */
-                j = TLVGet (re->tlv, 205);
+            case ROSTER_TYPE_ICQTIC:
+                j = TLVGet (re->tlv, TLV_ICQTIC);
                 if (j != (UWORD)-1)
                     s_repl (&roster->ICQTIC, re->tlv[j].str.txt);
                 break;
-            case 4: /* visibility */
+            case ROSTER_TYPE_VISIBILITY:
+            case ROSTER_TYPE_PRESENCE:
+            case ROSTER_TYPE_NONCONT:
+                /* TLV_UNKNIDLE   */
+                /* TLV_PRIVACY    */
+                /* TLV_VISIBILITY */
+                /* TLV_ALLOWIDLE  */
                 break;
             default:
                 rl_printf ("#Unknown type %d: %s %d %d.\n", re->type, re->name, re->tag, re->id);
@@ -265,10 +281,10 @@ JUMP_SNAC_F(SnacSrvReplyroster)
         {
             time_t now = time (NULL);
             UDWORD da = now;
-            SnacCliRosterentryadd (serv, "ImportTime", 0, 1, 19, 212, &da, 4);
+            SnacCliRosterentryadd (serv, "ImportTime", 0, 1, ROSTER_TYPE_IMPORTT, TLV_IMPORT, &da, 4);
         }
         if (!roster->ICQTIC)
-            SnacCliRosterentryadd (serv, "ICQTIC", 0, 2, 9, 205, "3608,0,0,0,60,null", 18);
+            SnacCliRosterentryadd (serv, "ICQTIC", 0, 2, ROSTER_TYPE_ICQTIC, TLV_ICQTIC, "3608,0,0,0,60,null", 18);
         event2->callback (event2);
         if (ContactGroupPrefVal (serv->contacts, CO_OBEYSBL))
         {
@@ -289,17 +305,28 @@ JUMP_SNAC_F(SnacSrvReplyroster)
 /*
  * CLI_ADDBUDDY - SNAC(13,8)
  */
+
+static void SnacCliRosterbulkone (Connection *serv, Contact *cont, Packet *pak, UWORD type)
+{
+    PacketWriteStrB     (pak, s_sprintf ("%ld", cont->uin));
+    PacketWriteB2       (pak, 0);
+    PacketWriteB2       (pak, ContactIDGet (cont, type));
+    PacketWriteB2       (pak, type);
+    PacketWriteBLen     (pak);
+    PacketWriteBLenDone (pak);
+    QueueEnqueueData (serv, QUEUE_CHANGE_ROSTER, pak->ref, 0x7fffffffL, NULL, cont, NULL, NULL);
+}
+
 void SnacCliRosterbulkadd (Connection *serv, ContactGroup *cs)
 {
     Packet *pak;
     Contact *cont;
     ContactGroup *cg;
-    UWORD type;
     int i;
     
     for (i = 0; (cont = ContactIndex (cs, i)); i++)
     {
-        if (cont->group && !ContactGroupPrefVal (cont->group, CO_ISSBL))
+        if (cont->group && cont->group->serv && !ContactGroupPrefVal (cont->group, CO_ISSBL))
         {
             SnacCliRosteradd (serv, cont->group, NULL);
             OptSetVal (&cont->group->copts, CO_ISSBL, 1);
@@ -319,30 +346,38 @@ void SnacCliRosterbulkadd (Connection *serv, ContactGroup *cs)
             SnacSend (serv, pak);
             pak = SnacC (serv, 19, 8, 0, 0);
         }
-        
-        if (ContactPrefVal (cont, CO_HIDEFROM))
-            type = 3;
-        else if (ContactPrefVal (cont, CO_INTIMATE))
-            type = 2;
-        else
-            type = 0;
 
-/*        SnacCliGrantauth (serv, cont);  */
+        
         PacketWriteStrB     (pak, s_sprintf ("%ld", cont->uin));
         PacketWriteB2       (pak, ContactGroupID (cg));
-        PacketWriteB2       (pak, ContactID (cont));
-        PacketWriteB2       (pak, type);
+        PacketWriteB2       (pak, ContactIDGet (cont, ROSTER_TYPE_NORMAL));
+        PacketWriteB2       (pak, ROSTER_TYPE_NORMAL);
         PacketWriteBLen     (pak);
-        PacketWriteTLVStr   (pak, 305, cont->nick);
+        PacketWriteTLVStr   (pak, TLV_NICK, cont->nick);
         if (cont->oldflags & CONT_REQAUTH)
         {
-            PacketWriteTLV     (pak, 102);
+            PacketWriteTLV     (pak, TLV_REQAUTH);
             PacketWriteTLVDone (pak);
         }
         PacketWriteBLenDone (pak);
         QueueEnqueueData (serv, QUEUE_CHANGE_ROSTER, pak->ref, 0x7fffffffL, NULL, cont, NULL, NULL);
-    }
 
+        if (ContactPrefVal (cont, CO_HIDEFROM))
+        {
+            i++;
+            SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_INVISIBLE);
+        }
+        if (ContactPrefVal (cont, CO_INTIMATE))
+        {
+            i++;
+            SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_VISIBLE);
+        }
+        if (ContactPrefVal (cont, CO_IGNORE))
+        {
+            i++;
+            SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_IGNORE);
+        }
+    }
     SnacSend (serv, pak);
     SnacCliAddend (serv);
 }
@@ -359,32 +394,41 @@ void SnacCliRosteradd (Connection *serv, ContactGroup *cg, Contact *cont)
         UWORD type = 0;
         
         if (!ContactGroupPrefVal (cg, CO_ISSBL))
+        {
             SnacCliRosteradd (serv, cg, NULL);
+            OptSetVal (&cg->copts, CO_ISSBL, 1);
+        }
         
+        SnacCliAddstart (serv);
+
         if (ContactPrefVal (cont, CO_HIDEFROM))
             type = 3;
         else if (ContactPrefVal (cont, CO_INTIMATE))
             type = 2;
 
-/*        SnacCliGrantauth (serv, cont);  */
-
-        SnacCliAddstart (serv);
         
         pak = SnacC (serv, 19, 8, 0, 0);
         PacketWriteStrB     (pak, s_sprintf ("%ld", cont->uin));
         PacketWriteB2       (pak, ContactGroupID (cg));
-        PacketWriteB2       (pak, ContactID (cont));
+        PacketWriteB2       (pak, ContactIDGet (cont, ROSTER_TYPE_NORMAL));
         PacketWriteB2       (pak, type);
         PacketWriteBLen     (pak);
-        PacketWriteTLVStr   (pak, 305, cont->nick);
+        PacketWriteTLVStr   (pak, TLV_NICK, cont->nick);
         if (cont->oldflags & CONT_REQAUTH)
         {
-            PacketWriteTLV     (pak, 102);
+            PacketWriteTLV     (pak, TLV_REQAUTH);
             PacketWriteTLVDone (pak);
         }
         PacketWriteBLenDone (pak);
         QueueEnqueueData (serv, QUEUE_CHANGE_ROSTER, pak->ref, 0x7fffffffL, NULL, cont, NULL, NULL);
 
+        if (ContactPrefVal (cont, CO_HIDEFROM))
+            SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_INVISIBLE);
+        if (ContactPrefVal (cont, CO_INTIMATE))
+            SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_VISIBLE);
+        if (ContactPrefVal (cont, CO_IGNORE))
+            SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_IGNORE);
+
         SnacSend (serv, pak);
         SnacCliAddend (serv);
     }
@@ -394,7 +438,7 @@ void SnacCliRosteradd (Connection *serv, ContactGroup *cg, Contact *cont)
         PacketWriteStrB     (pak, cg->name);
         PacketWriteB2       (pak, ContactGroupID (cg));
         PacketWriteB2       (pak, 0);
-        PacketWriteB2       (pak, 1);
+        PacketWriteB2       (pak, ROSTER_TYPE_GROUP);
         PacketWriteBLen     (pak);
         PacketWriteBLenDone (pak);
         SnacSend (serv, pak);
@@ -439,22 +483,29 @@ void SnacCliRosterupdate (Connection *serv, ContactGroup *cg, Contact *cont)
 
         PacketWriteStrB     (pak, s_sprintf ("%ld", cont->uin));
         PacketWriteB2       (pak, ContactGroupID (cg));
-        PacketWriteB2       (pak, ContactID (cont));
+        PacketWriteB2       (pak, ContactIDGet (cont, ROSTER_TYPE_NORMAL));
         PacketWriteB2       (pak, type);
         PacketWriteBLen     (pak);
-        PacketWriteTLVStr   (pak, 305, cont->nick);
+        PacketWriteTLVStr   (pak, TLV_NICK, cont->nick);
         PacketWriteBLenDone (pak);
+
+        if (ContactPrefVal (cont, CO_HIDEFROM))
+            SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_INVISIBLE);
+        if (ContactPrefVal (cont, CO_INTIMATE))
+            SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_VISIBLE);
+        if (ContactPrefVal (cont, CO_IGNORE))
+            SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_IGNORE);
     }
     else
     {
         PacketWriteStrB     (pak, cg->name);
         PacketWriteB2       (pak, ContactGroupID (cg));
         PacketWriteB2       (pak, 0);
-        PacketWriteB2       (pak, 1);
+        PacketWriteB2       (pak, ROSTER_TYPE_GROUP);
         PacketWriteBLen     (pak);
-        PacketWriteTLV      (pak, 200);
+        PacketWriteTLV      (pak, TLV_GROUPITEMS);
         for (i = 0; (cont = ContactIndex (cg, i)); i++)
-            PacketWriteB2   (pak, ContactID (cont));
+            PacketWriteB2   (pak, ContactIDGet (cont, ROSTER_TYPE_NORMAL));
         PacketWriteTLVDone  (pak);
         PacketWriteBLenDone (pak);
     }
@@ -475,7 +526,7 @@ void SnacCliSetvisibility (Connection *serv)
     PacketWriteB2       (pak, 4);
     PacketWriteBLen     (pak);
     PacketWriteB2       (pak, 5);
-    PacketWriteTLV      (pak, 202);
+    PacketWriteTLV      (pak, TLV_PRIVACY);
     PacketWrite1        (pak, 4);
     PacketWriteTLVDone  (pak);
     PacketWriteBLenDone (pak);
@@ -485,30 +536,55 @@ void SnacCliSetvisibility (Connection *serv)
 /*
  * CLI_ROSTERDELETE - SNAC(13,a)
  */
-void SnacCliRosterdelete (Connection *serv, ContactGroup *cg, Contact *cont)
+void SnacCliRosterdeletegroup (Connection *serv, ContactGroup *cg)
 {
     Packet *pak;
 
+    if (!cg->id)
+        return;
+
     SnacCliAddstart (serv);
     pak = SnacC (serv, 19, 10, 0, 0);
-    if (cont)
+    PacketWriteStrB     (pak, cg->name);
+    PacketWriteB2       (pak, ContactGroupID (cg));
+    PacketWriteB2       (pak, 0);
+    PacketWriteB2       (pak, ROSTER_TYPE_GROUP);
+    PacketWriteBLen     (pak);
+    PacketWriteBLenDone (pak);
+    SnacSend (serv, pak);
+    SnacCliAddend (serv);
+}
+
+/*
+ * CLI_ROSTERDELETE - SNAC(13,a)
+ */
+void SnacCliRosterdeletecontact (Connection *serv, Contact *cont)
+{
+    Packet *pak;
+    ContactIDs *ids;
+    
+    if (!cont->ids)
+        return;
+    
+    SnacCliAddstart (serv);
+    pak = SnacC (serv, 19, 10, 0, 0);
+
+    if ((ids = ContactID (cont, ROSTER_TYPE_NORMAL)) && ids->issbl)
     {
         PacketWriteStrB     (pak, s_sprintf ("%ld", cont->uin));
-        PacketWriteB2       (pak, ContactGroupID (cg));
-        PacketWriteB2       (pak, ContactID (cont));
-        PacketWriteB2       (pak, 0);
-        PacketWriteBLen     (pak);
-        PacketWriteBLenDone (pak);
-    }
-    else
-    {
-        PacketWriteStrB     (pak, cg->name);
-        PacketWriteB2       (pak, ContactGroupID (cg));
-        PacketWriteB2       (pak, 0);
-        PacketWriteB2       (pak, 1);
+        PacketWriteB2       (pak, ids->tag);
+        PacketWriteB2       (pak, ids->id);
+        PacketWriteB2       (pak, ROSTER_TYPE_NORMAL);
         PacketWriteBLen     (pak);
         PacketWriteBLenDone (pak);
     }
+    if ((ids = ContactID (cont, ROSTER_TYPE_INVISIBLE)) && ids->issbl)
+        SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_INVISIBLE);
+    if ((ids = ContactID (cont, ROSTER_TYPE_VISIBLE)) && ids->issbl)
+        SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_VISIBLE);
+    if ((ids = ContactID (cont, ROSTER_TYPE_IGNORE)) && ids->issbl)
+        SnacCliRosterbulkone (serv, cont, pak, ROSTER_TYPE_IGNORE);
+
     SnacSend (serv, pak);
     SnacCliAddend (serv);
 }
@@ -576,8 +652,6 @@ JUMP_SNAC_F(SnacSrvUpdateack)
             case 3:
                 if (cont)
                 {
-                    cont->id = 0;
-                    cont->tag = 0;
                     rl_printf (i18n (2566, "Contact upload for %s%s%s failed, already on server.\n"),
                                COLCONTACT, s_quote (cont->nick), COLNONE);
                 }
@@ -648,10 +722,10 @@ JUMP_SNAC_F(SnacSrvRosterok)
     {
         time_t now = time (NULL);
         UDWORD da = now;
-        SnacCliRosterentryadd (serv, "ImportTime", 0, 1, 19, 212, &da, 4);
+        SnacCliRosterentryadd (serv, "ImportTime", 0, 1, ROSTER_TYPE_IMPORTT, TLV_IMPORT, &da, 4);
     }
     if (!roster->ICQTIC)
-        SnacCliRosterentryadd (serv, "ICQTIC", 0, 2, 9, 205, "3608,0,0,0,60,null", 18);
+        SnacCliRosterentryadd (serv, "ICQTIC", 0, 2, ROSTER_TYPE_ICQTIC, TLV_ICQTIC, "3608,0,0,0,60,null", 18);
     event2->callback (event2);
     if (ContactGroupPrefVal (serv->contacts, CO_OBEYSBL))
     {
