@@ -1,4 +1,4 @@
-/* $Id: server.c,v 1.80 2005/02/22 22:10:21 kuhlmann Exp $ */
+/* $Id: server.c,v 1.81 2005/03/31 17:28:15 kuhlmann Exp $ */
 /* Copyright: This file may be distributed under version 2 of the GPL licence. */
 
 #include "micq.h"
@@ -44,6 +44,27 @@ static void CallbackTogvis (Event *event)
 }
 
 UBYTE IMCliMsg (Connection *conn, Contact *cont, Opt *opt)
+{
+    const char *opt_text;
+    UDWORD opt_type, opt_trans;
+    
+    if (!cont || !conn || !OptGetStr (opt, CO_MSGTEXT, &opt_text))
+    {
+        OptD (opt);
+        return RET_FAIL;
+    }
+    if (!OptGetVal (opt, CO_MSGTYPE, &opt_type))
+        OptSetVal (opt, CO_MSGTYPE, opt_type = MSG_NORM);
+    if (!OptGetVal (opt, CO_MSGTRANS, &opt_trans))
+        OptSetVal (opt, CO_MSGTRANS, opt_trans = CV_MSGTRANS_ANY);
+    
+    putlog (conn, NOW, cont, STATUS_ONLINE, 
+            opt_type == MSG_AUTO ? LOG_AUTO : LOG_SENT, opt_type, opt_text);
+
+    return IMCliReMsg (conn, cont, opt);
+}
+
+UBYTE IMCliReMsg (Connection *conn, Contact *cont, Opt *opt)
 {
     const char *opt_text;
     Event *event;
@@ -72,9 +93,6 @@ UBYTE IMCliMsg (Connection *conn, Contact *cont, Opt *opt)
         QueueEnqueueData (conn, QUEUE_TOGVIS, 0, time (NULL) + reveal_time, NULL, cont, NULL, CallbackTogvis);
     }
 
-    putlog (conn, NOW, cont, STATUS_ONLINE, 
-            opt_type == MSG_AUTO ? LOG_AUTO : LOG_SENT, opt_type, opt_text);
-
 #ifdef ENABLE_PEER2PEER
     if (opt_trans & CV_MSGTRANS_DC)
         if (conn->assoc)
