@@ -26,11 +26,12 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: util_rl.c,v 1.40 2005/04/26 21:57:07 kuhlmann Exp $
+ * $Id: util_rl.c,v 1.44 2005/05/24 21:27:53 kuhlmann Exp $
  */
 
 #include "micq.h"
 #include "util_rl.h"
+#include "util_alias.h"
 #include "tabs.h"
 #include "conv.h"
 #include "contact.h"
@@ -82,7 +83,14 @@
 
 #undef DEBUG_RL
 
-/* We'd like to use win_t and wchar_t to store the input line,
+#if defined(_WIN32) && !defined(__CYGWIN__)
+#define ANSI_CLEAR ""
+#else
+#define ANSI_CLEAR ESC "[J"
+#endif
+
+
+/* We'd like to use wint_t and wchar_t to store the input line,
    however not all C implementations store the unicode codepoint
    in wint_t/wchar_t, which kills the possibility to enter an
    arbitrary codepoint into the line. Thus, we end up using UDWORDS instead. */
@@ -131,7 +139,7 @@ static void rl_key_end (void);
 static void rl_key_cut (void);
 static void rl_key_kill (void);
 static void rl_linecompress (str_t line, UDWORD from, UDWORD to);
-static void rl_lineexpand (char *hist);
+static void rl_lineexpand (const char *hist);
 static void rl_historyback (void);
 static void rl_historyforward (void);
 static void rl_historyadd (void);
@@ -174,14 +182,6 @@ static UWORD  rl_prompt_len = 0;
 static int    rl_prompt_stat = 0;
 static time_t rl_prompt_time = 0;
 
-typedef struct rl_autoexpandv_s {
-  struct rl_autoexpandv_s *next;
-  char *command;
-  char *replace;
-} rl_autoexpandv_t;
-      
-static rl_autoexpandv_t *rl_ae = NULL;
-
 /*
  * Initialize read line code
  */
@@ -444,7 +444,7 @@ static int rl_getcolumns ()
 void ReadLineClrScr ()
 {
 #ifdef ANSI_TERM
-    printf (ESC "[H" ESC "[J");
+    printf (ESC "[H" ANSI_CLEAR);
 #else
     printf ("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
 #endif
@@ -561,7 +561,7 @@ static void rl_recheck (BOOL clear)
 #ifdef ANSI_TERM
     s_cat (&rl_operate, " \b");
     if (clear)
-        s_cat (&rl_operate, ESC "[J");
+        s_cat (&rl_operate, ANSI_CLEAR);
 #else
     s_cat (&rl_operate, "     \b\b\b\b\b");
 #endif
@@ -837,7 +837,7 @@ static void rl_linecompress (str_t line, UDWORD from, UDWORD to)
 /*
  * Expand given UTF8 string into (and replace) editing line
  */
-static void rl_lineexpand (char *hist)
+static void rl_lineexpand (const char *hist)
 {
     str_s str = { NULL, 0, 0 };
     int off;
@@ -848,7 +848,7 @@ static void rl_lineexpand (char *hist)
     s_init (&rl_display, "", 0);
     rl_colpos = rl_ucspos = rl_bytepos = 0;
     
-    str.txt = hist;
+    str.txt = (char *) hist;
     str.len = strlen (str.txt);
     for (off = 0; off < str.len; )
         rl_insert (ConvGetUTF8 (&str, &off));
@@ -1413,68 +1413,21 @@ static void rl_key_kill (void)
 
 /*** read line auto expander ***/
 
-const rl_autoexpand_t *ReadLineListAutoExpand (void)
-{
-    return (const rl_autoexpand_t *) rl_ae;
-}
-
-void ReadLineAutoExpand (const char *command, const char *replace)
-{
-    rl_autoexpandv_t *e;
-    
-    for (e = rl_ae; e; e = e->next)
-    {
-        if (!strcmp (command, e->command))
-        {
-            s_repl (&e->replace, replace);
-            return;
-        }
-    }
-    e = malloc (sizeof (rl_autoexpandv_t));
-    e->next = rl_ae;
-    e->command = strdup (command);
-    e->replace = strdup (replace);
-    rl_ae = e;
-}
-
 static void rl_checkautoexpand (void)
 {
-    rl_autoexpandv_t *e;
-    static str_s rl_exp = { NULL, 0, 0 };
-    char *p, *q;
+    const char *replacement;
     
+    if (!rl_bytepos)
+        return;
+    
     rl_linecompress (&rl_temp, 0, -1);
-        
-    for (e = rl_ae; e; e = e->next)
-        if (!strcmp (rl_temp.txt, e->command))
-            break;
-
-    if (!e)
+    replacement = AliasExpand (rl_temp.txt, rl_bytepos, TRUE);
+    if (!replacement)
         return;
     
-    s_init (&rl_exp, "", 0);
-    for (p = e->replace; *p && (q = strchr (p, '%')); p = q)
-    {
-        s_catn (&rl_exp, p, q - p);
-        if (q[1] == 'r')
-        {
-            if (uiG.last_rcvd)
-                s_cat (&rl_exp, uiG.last_rcvd->nick ? uiG.last_rcvd->nick : s_sprintf ("%lu", uiG.last_rcvd->uin));
-            q += 2;
-        }
-        else if (q[1] == 'a')
-        {
-            if (uiG.last_sent)
-                s_cat (&rl_exp, uiG.last_sent->nick ? uiG.last_sent->nick : s_sprintf ("%lu", uiG.last_sent->uin));
-            q += 2;
-        }
-        else
-            q++;
-    }
-    s_cat (&rl_exp, p);
     rl_tab_state = 0;
     rl_goto (0);
-    rl_lineexpand (rl_exp.txt);
+    rl_lineexpand (replacement);
 }
 
 /*** read line input processor ***/
@@ -1568,6 +1521,8 @@ str_t ReadLine (UBYTE newbyte)
                 case '\r':
                 case '\n':
                     rl_tab_accept ();
+                    if (rl_tab_state == 0)
+                        rl_checkautoexpand ();
                     rl_key_end ();
                     rl_historyadd ();
                     break;
@@ -1590,7 +1545,7 @@ str_t ReadLine (UBYTE newbyte)
                 case 32:             /*   = SPACE */
                     if (rl_tab_state > 0)
                         rl_tab_accept ();
-                    else
+                    else if (rl_tab_state == 0)
                         rl_checkautoexpand ();
                     rl_insert (' ');
                     break;
@@ -1804,7 +1759,7 @@ void ReadLinePrompt ()
     {
         s_catc (&rl_operate, '\r');
 #ifdef ANSI_TERM
-        s_cat (&rl_operate, ESC "[J");
+        s_cat (&rl_operate, ANSI_CLEAR);
 #endif
         rl_prompt_stat = 0;
     }
@@ -1839,7 +1794,7 @@ void ReadLinePromptHide ()
     rl_goto (0);
     s_catc (&rl_operate, '\r');
 #ifdef ANSI_TERM
-    s_cat (&rl_operate, ESC "[J");
+    s_cat (&rl_operate, ANSI_CLEAR);
 #endif
     printf ("%s", rl_operate.txt);
     rl_prompt_stat = 0;
