@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: util_ssl.c,v 1.22 2005/02/27 19:35:59 kuhlmann Exp $
+ * $Id: util_ssl.c,v 1.24 2005/04/09 13:45:03 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -524,7 +524,11 @@ int ssl_handshake (Connection *conn DEBUGPARAM)
     if (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED)
     {
         conn->ssl_status = SSL_STATUS_HANDSHAKE;
-        conn->connect |= CONNECT_SELECT_R | CONNECT_SELECT_W;
+        conn->connect &= ~CONNECT_SELECT_R & ~CONNECT_SELECT_W;
+        if (gnutls_record_get_direction (conn->ssl))
+           conn->connect |= CONNECT_SELECT_W;
+        else
+           conn->connect |= CONNECT_SELECT_R;
         return 1;
     }
 #else
